"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _defaults3 = _interopRequireDefault(require("lodash/defaults"));

var _apolloClient = require("apollo-client");

var _apolloLink = require("apollo-link");

var _apolloUtilities = require("apollo-utilities");

var _isBrowser = _interopRequireDefault(require("./isBrowser"));

var _Authenticator = _interopRequireDefault(require("./Authenticator"));

var _ErrorLink = _interopRequireDefault(require("./links/ErrorLink"));

var _RetryLink = _interopRequireDefault(require("./links/RetryLink"));

var _AuthLink = _interopRequireDefault(require("./links/AuthLink"));

var _HttpLink = _interopRequireDefault(require("./links/HttpLink"));

var _WebSocketLink = _interopRequireDefault(require("./links/WebSocketLink"));

var _ApolloListener = _interopRequireDefault(require("./ApolloListener"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } return _assertThisInitialized(self); }

function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); if (superClass) _setPrototypeOf(subClass, superClass); }

function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function createLink(configs, auth) {
  var uri = configs.uri;
  var httpLink = new _HttpLink.default(_objectSpread({
    uri: uri
  }, configs.http), auth);
  if (!_isBrowser.default) return httpLink;
  var wsLink = new _WebSocketLink.default(_objectSpread({
    uri: uri
  }, configs.ws), auth);
  return _apolloLink.ApolloLink.split(function (_ref) {
    var query = _ref.query;

    var _getMainDefinition = (0, _apolloUtilities.getMainDefinition)(query),
        kind = _getMainDefinition.kind,
        operation = _getMainDefinition.operation;

    return kind === "OperationDefinition" && operation === "subscription";
  }, wsLink, httpLink);
}

function get(target, name) {
  if (typeof _Authenticator.default.prototype[name] === "function") {
    return target.auth[name].bind(target.auth);
  }

  return target[name];
}

var _default =
/*#__PURE__*/
function (_ApolloClient) {
  _inherits(_default, _ApolloClient);

  function _default(options) {
    var _this;

    _classCallCheck(this, _default);

    var _defaults2 = (0, _defaults3.default)({}, options, {
      uri: "http://localhost:3000/",
      onError: function onError() {}
    }),
        cache = _defaults2.cache,
        connectToDevTools = _defaults2.connectToDevTools,
        configs = _objectWithoutProperties(_defaults2, ["cache", "connectToDevTools"]);

    var uri = configs.uri;
    var auth = new _Authenticator.default(_objectSpread({
      uri: uri
    }, configs.auth));
    _this = _possibleConstructorReturn(this, _getPrototypeOf(_default).call(this, {
      connectToDevTools: connectToDevTools,
      ssrMode: !_isBrowser.default,
      link: _apolloLink.ApolloLink.from([new _ErrorLink.default(configs.onError, auth), new _AuthLink.default(configs.auth, auth), new _RetryLink.default(configs.retry, auth), createLink(configs, auth)]),
      cache: cache
    }));
    _this.auth = auth;
    _this.configs = configs;

    var resetStore = function resetStore() {
      return _this.resetStore();
    };

    _ApolloListener.default.on("signInResolve", resetStore);

    _ApolloListener.default.on("signOutResolve", resetStore);

    _ApolloListener.default.on("changeViewerResolve", resetStore);

    return _possibleConstructorReturn(_this, new Proxy(_assertThisInitialized(_this), {
      get: get
    }));
  }

  return _default;
}(_apolloClient.ApolloClient);

exports.default = _default;