import "isomorphic-unfetch";
import _ from "lodash";
import fs from "fs";
import http from "http";
import gql from "graphql-tag";
import express from "express";
import { ApolloServer, PubSub } from "apollo-server-express";
import { JsonWebToken } from "../../server";

export const pubsub = new PubSub();

export const onSignIn = jest.fn(() => ({ correlationId: "NEW" }));
export const onSignOut = jest.fn(() => ({ correlationId: "NEW" }));
export const onRenew = jest.fn(() => ({ correlationId: "NEW" }));
export const onChangeViewer = jest.fn(() => ({ correlationId: "NEW" }));
export const resolve = jest.fn(() => 1);
export const context = jest.fn(() => ({}));

const typeDefs = gql`
  type Query {
    viewer: Int
  }
  type Subscription {
    viewer: Int
  }
`;

const resolvers = {
  Query: {
    viewer: () => resolve(),
  },
  Subscription: {
    viewer: {
      subscribe: () => {
        setTimeout(() => pubsub.publish("NEW_SUBSCRIPTION"), 200);
        return pubsub.asyncIterator(["NEW_VIEWER"]);
      },
    },
  },
};

const app = express();

export const jwt = new JsonWebToken(
  { private: fs.readFileSync(`${__dirname}/../../../__mocks__/private.pem`) },
  { onSignIn, onSignOut, onRenew },
);

jwt.applyMiddleware(app);

const apolloServer = new ApolloServer({
  typeDefs,
  resolvers,
  context: ({ req, connection }) => {
    const headers = _.get(req, ["headers"]) || _.get(connection, ["context"]);
    return context(headers);
  },
});

apolloServer.applyMiddleware({ app });

const server = http.createServer(app);
apolloServer.installSubscriptionHandlers(server);

export default {
  listen: () =>
    new Promise(r =>
      server.listen(() => {
        const { port } = server.address();
        r(`http://localhost:${port}`);
      }),
    ),
  close: () => new Promise(r => server.close(r)),
};
