/* eslint-disable import/first */

jest.mock("../isBrowser", () => ({ default: true }));

import "isomorphic-unfetch";
import React from "react";
import ws from "ws";
import { ApolloProvider, Query } from "react-apollo";
import gql from "graphql-tag";
import { shallow, diveTo } from "enzyme";
import PropTypes from "prop-types";
import { withProvider } from "..";
import server from "../__mocks__/server";

describe("withApollo on browser", () => {
  let Main;
  let Apollo;
  let wrapApollo;

  beforeAll(async () => {
    const uri = await server.listen();
    wrapApollo = withProvider({ uri, ws: { webSocketImpl: ws } });
  });

  afterAll(async () => server.close());

  beforeEach(() => {
    Main = class extends React.Component {
      static propTypes = {
        apolloClient: PropTypes.shape().isRequired,
      };

      static childContextTypes = {
        client: PropTypes.shape(),
      };

      getChildContext() {
        const { apolloClient } = this.props;
        return { client: apolloClient };
      }

      render() {
        const { apolloClient } = this.props;
        return (
          <ApolloProvider client={apolloClient}>
            <Query
              query={gql`
                query {
                  viewer
                }
              `}
            >
              {({ data }) => <div>{data.viewer}</div>}
            </Query>
          </ApolloProvider>
        );
      }
    };

    Apollo = jest.fn(props => <Main {...props} />);
  });

  it("successfully render", async () => {
    const App = Apollo |> wrapApollo;
    const props = await App.getInitialProps();
    expect(props).toEqual({ apolloState: {} });
    expect(diveTo(shallow(<App {...props} />), "div").html()).toBe(
      "<div></div>",
    );
    await new Promise(next => setTimeout(next, 500));
    expect(diveTo(shallow(<App {...props} />), "div").html()).toBe(
      "<div>1</div>",
    );
    expect(Apollo.mock.calls[0]).toEqual(Apollo.mock.calls[1]);
  });
});
