import "isomorphic-unfetch";
import React from "react";
import ws from "ws";
import { ApolloProvider, Query } from "react-apollo";
import gql from "graphql-tag";
import { shallow, diveTo } from "enzyme";
import PropTypes from "prop-types";
import { withProvider } from "..";
import server, { resolve } from "../__mocks__/server";

describe("withApollo on server", () => {
  let Main;
  let Apollo;
  let wrapApollo;

  beforeAll(async () => {
    const uri = await server.listen();
    wrapApollo = withProvider({ uri, ws: { webSocketImpl: ws } });
  });

  afterAll(async () => server.close());

  beforeEach(() => {
    Main = class extends React.Component {
      static propTypes = {
        apolloClient: PropTypes.shape().isRequired,
      };

      static childContextTypes = {
        client: PropTypes.shape(),
      };

      getChildContext() {
        const { apolloClient } = this.props;
        return { client: apolloClient };
      }

      render() {
        const { apolloClient } = this.props;

        return (
          <ApolloProvider client={apolloClient}>
            <Query
              query={gql`
                query {
                  viewer
                }
              `}
            >
              {({ data }) => <div>{data.viewer}</div>}
            </Query>
          </ApolloProvider>
        );
      }
    };

    Apollo = jest.fn(props => <Main {...props} />);
  });

  it("successfully get data", async () => {
    const App = Apollo |> wrapApollo;
    const props = await App.getInitialProps();
    expect(props).toEqual({ apolloState: { ROOT_QUERY: { viewer: 1 } } });
    expect(diveTo(shallow(<App {...props} />), "div").html()).toBe(
      "<div>1</div>",
    );
    expect(resolve).toHaveBeenCalledTimes(1);
  });

  it("successfully get data with getInitialProps", async () => {
    Apollo.getInitialProps = () => ({ name: "nextjs-apollo" });

    const App = Apollo |> wrapApollo;
    const props = await App.getInitialProps();
    expect(props).toEqual({
      apolloState: { ROOT_QUERY: { viewer: 1 } },
      name: "nextjs-apollo",
    });
    expect(diveTo(shallow(<App {...props} />), "div").html()).toBe(
      "<div>1</div>",
    );
  });

  it("when render error", async () => {
    console.error = jest.fn();  // eslint-disable-line
    const Component = () => {
      throw new Error("render error");
    };
    const App = Component |> withProvider();
    await App.getInitialProps();
    expect(console.error).toHaveBeenCalledWith(  // eslint-disable-line
      "Error while running `getDataFromTree`",
      "render error",
    );
  });
});
