import { Observable, ApolloLink } from "apollo-link";
import ApolloToken from "../ApolloToken";

export default class extends ApolloLink {
  constructor(options, auth) {
    super((operation, forward) => {
      if (
        ApolloToken.refreshToken &&
        (Number(ApolloToken.expired) || 0) < Date.now()
      ) {
        return new Observable(async observer => {
          try {
            await auth.renew();
          } catch (e) {
            // empty
          }

          forward(operation).subscribe({
            next: observer.next.bind(observer),
            error: observer.error.bind(observer),
            complete: observer.complete.bind(observer),
          });
        });
      }

      return forward(operation);
    });
  }
}
