import _ from "lodash";
import React from "react";
import PropTypes from "prop-types";
import wrapDisplayName from "recompose/wrapDisplayName";
import { assertResult } from "thelper";
import withConsumer from "./withConsumer";
import ApolloListener from "./ApolloListener";
import ApolloToken from "./ApolloToken";

export default (redirect, options) => {
  const { disabled } = _.defaults({}, options);
  assertResult(redirect, new TypeError("redirect is required"));

  const redirectTo = () => {
    window.location = redirect;
  };

  return Component => {
    class withAuthorized extends React.Component {
      static displayName = wrapDisplayName(Component, "withAuthorized");

      static propTypes = {
        client: PropTypes.shape().isRequired,
      };

      constructor(props) {
        super(props);
        const authorized = disabled || ApolloToken.expired > Date.now();
        this.state = { authorized };
      }

      componentDidMount() {
        const { authorized } = this.state;
        this.listeners = [
          ApolloListener.on("renewRejected", redirectTo),
          ApolloListener.on("signOut", redirectTo),
        ];
        if (!authorized) this.renewAuthorized();
      }

      componentWillUnmount() {
        _.forEach(this.listeners, unListener => unListener());
      }

      async renewAuthorized() {
        try {
          const { client } = this.props;

          if (!ApolloToken.refreshToken) throw new Error();

          await client.renew();
          this.setState({ authorized: true });
        } catch (e) {
          redirectTo();
        }
      }

      render() {
        const { authorized } = this.state;
        if (authorized) {
          return React.createElement(Component, this.props);
        }
        return null;
      }
    }

    return withAuthorized |> withConsumer();
  };
};
