import _ from "lodash";
import fp from "lodash/fp";
import React from "react";
import { withProps, setDisplayName } from "recompose";
import { Mutation } from "react-apollo";
import getFieldName from "./helps/getFieldName";

const mutationProps = [
  "mutation",
  "variables",
  "update",
  "ignoreResults",
  "optimisticResponse",
  "refetchQueries",
  "awaitRefetchQueries",
  "onCompleted",
  "onError",
  "context",
];

export function format(name, { fieldName, result, parent, mutate }) {
  const { results } = parent;

  return {
    ...result,
    [name]: async (input, options) => {
      const { data } = await mutate({
        variables: { input },
        ...options,
      });
      return data[fieldName];
    },
    results: { ...results, [name]: result },
  };
}

export default (name, mutation) => Component => {
  const fieldName = getFieldName(name, mutation);
  return fp.compose(
    setDisplayName(_.camelCase(["withMutation", name])),
    withProps((parent = {}) => ({
      mutation,
      ignoreResults: false,
      children: (mutate, result = {}) => {
        const props = _.omit(parent, mutationProps);
        const apolloProps = format(name, { fieldName, result, parent, mutate });
        return <Component {...props} {...apolloProps} />;
      },
    })),
  )(Mutation);
};
