import React from "react";
import Head from "next/head";
import PropTypes from "prop-types";
import { getDataFromTree } from "react-apollo";
import thunk from "thelper/thunk";
import initApollo from "./initApollo";
import isBrowser from "./isBrowser";

export default (options = {}) => {
  const thunkOption = thunk(options);

  return App =>
    class extends React.Component {
      static displayName = "withProvider(App)";

      static propTypes = {
        apolloState: PropTypes.shape().isRequired,
      };

      static async getInitialProps(ctx = {}) {
        const { Component, router } = ctx;

        const appProps = App.getInitialProps
          ? await App.getInitialProps(ctx)
          : {};

        const apolloClient = initApollo({}, thunkOption(appProps));

        if (!isBrowser) {
          try {
            await getDataFromTree(
              <App
                {...appProps}
                Component={Component}
                router={router}
                apolloClient={apolloClient}
              />,
            );
          } catch (error) {
            console.error('Error while running `getDataFromTree`', error.message); // eslint-disable-line
          }

          Head.rewind();
        }

        const apolloState = apolloClient.cache.extract();

        return { ...appProps, apolloState };
      }

      constructor(props) {
        super(props);
        this.apolloClient = initApollo(props.apolloState, thunkOption(props));
      }

      render() {
        return <App {...this.props} apolloClient={this.apolloClient} />;
      }
    };
};
