import _ from "lodash";
import fp from "lodash/fp";
import React from "react";
import { Query } from "react-apollo";
import { withProps, setDisplayName } from "recompose";
import getFieldName from "./helps/getFieldName";

const queryProps = [
  "query",
  "variables",
  "pollInterval",
  "notifyOnNetworkStatusChange",
  "fetchPolicy",
  "errorPolicy",
  "ssr",
  "displayName",
  "skip",
  "onCompleted",
  "onError",
  "context",
  "partialRefetch",
];

export function format(name, { fieldName, result, parent }) {
  const { results, skip, loading } = parent;

  return {
    ...result,
    results: { ...results, [name]: result },
    [name]: _.get(result, ["data", fieldName]),
    loading: (!skip && result.loading) || loading || false,
  };
}

export default (name, query) => Component => {
  const fieldName = getFieldName(name, query);
  return fp.compose(
    setDisplayName(_.camelCase(["withQuery", name])),
    withProps((parent = {}) => ({
      query,
      children: (result = {}) => {
        const props = _.omit(parent, queryProps);
        const apolloProps = format(name, { fieldName, result, parent });
        return <Component {...props} {...apolloProps} />;
      },
    })),
  )(Query);
};
