import _ from "lodash";
import fp from "lodash/fp";
import React from "react";
import { withProps, setDisplayName } from "recompose";
import { Subscription } from "react-apollo";

const subscriptionProps = [
  "subscription",
  "variables",
  "shouldResubscribe",
  "onSubscriptionData",
  "fetchPolicy",
];

export function format(name, result, parent) {
  const { results, loading } = parent;

  return {
    ...result,
    results: { ...results, [name]: result },
    [name]: _.get(result, ["data", name]),
    loading: result.loading || loading || false,
  };
}

export default (name, subscription) => Component =>
  fp.compose(
    setDisplayName(_.camelCase(["withSubscription", name])),
    withProps(parent => ({
      subscription,
      children: result => {
        const props = _.omit(parent, subscriptionProps);
        const apolloProps = format(name, result, parent);
        return <Component {...props} {...apolloProps} />;
      },
    })),
  )(Subscription);
