import _ from "lodash";
import fp from "lodash/fp";
import { assertResult } from "thelper";
import { withProps, setDisplayName } from "recompose";
import withMutation from "./withMutation";

function upload(file, { uri, data, progress }) {
  assertResult(uri, new TypeError("uri is required"));

  const form = new FormData();
  _.forEach(data, (value, name) => form.append(name, value));
  form.append("file", file);

  return new Promise((resolve, reject) => {
    const onprogress = event => {
      if (event.lengthComputable && progress) {
        progress(event.loaded / event.total);
      }
    };

    const xhr = new XMLHttpRequest();
    xhr.open("POST", uri);
    xhr.upload.onprogress = onprogress;
    xhr.onprogress = onprogress;
    xhr.onload = () => {
      if (_.inRange(xhr.status, 200, 300)) {
        resolve(JSON.parse(xhr.response));
        return;
      }

      reject(new Error("failed to upload: server error"));
    };
    xhr.onerror = () => reject(new Error("failed to upload: network error"));
    xhr.send(form);
  });
}

export default (name, mutation, configs) =>
  fp.compose(
    setDisplayName(_.camelCase(["withUploader", name])),
    withMutation(name, mutation),
    withProps(({ [name]: mutate }) => ({
      [name]: async (file, { input, progress, ...options } = {}) => {
        const settings = { ...configs, ...options };
        const data = await mutate(input, settings);
        return upload(file, { ...settings, data, progress });
      },
    })),
  );
